package java.net;

import sun.net.ConnectionResetException;
import java.io.FileDescriptor;
import java.nio.channels.FileChannel;
import java.io.IOException;
import java.io.FileInputStream;

class SocketInputStream extends FileInputStream
{
    private boolean eof;
    private AbstractPlainSocketImpl impl;
    private byte[] temp;
    private Socket socket;
    private boolean closing;
    
    SocketInputStream(AbstractPlainSocketImpl impl) throws IOException {
        super(impl.getFileDescriptor());
        this.impl = null;
        this.socket = null;
        this.closing = false;
        this.impl = impl;
        this.socket = impl.getSocket();
    }
    
    @Override
    public final FileChannel getChannel() {
        return null;
    }
    
    private native int socketRead0(FileDescriptor p0, byte[] p1, int p2, int p3, int p4) throws IOException;
    
    private int socketRead(FileDescriptor fileDescriptor, byte[] array, int n, int n2, int n3) throws IOException {
        return this.socketRead0(fileDescriptor, array, n, n2, n3);
    }
    
    @Override
    public int read(byte[] array) throws IOException {
        return this.read(array, 0, array.length);
    }
    
    @Override
    public int read(byte[] array, int n, int n2) throws IOException {
        return this.read(array, n, n2, this.impl.getTimeout());
    }
    
    int read(byte[] array, int i, int j, int n) throws IOException {
        if (this.eof) {
            return -1;
        }
        if (this.impl.isConnectionReset()) {
            throw new SocketException("Connection reset");
        }
        if (j <= 0 || i < 0 || j > array.length - i) {
            if (j == 0) {
                return 0;
            }
            throw new ArrayIndexOutOfBoundsException("length == " + j + " off == " + i + " buffer length == " + array.length);
        }
        else {
            boolean n2 = false;
            FileDescriptor acquireFD = this.impl.acquireFD();
            try {
                int socketRead = this.socketRead(acquireFD, array, i, j, n);
                if (socketRead > 0) {
                    return socketRead;
                }
            }
            catch (ConnectionResetException ex) {
                n2 = true;
            }
            finally {
                this.impl.releaseFD();
            }
            if (!(n2 == 0)) {
                this.impl.setConnectionResetPending();
                this.impl.acquireFD();
                try {
                    int socketRead2 = this.socketRead(acquireFD, array, i, j, n);
                    if (socketRead2 > 0) {
                        return socketRead2;
                    }
                }
                catch (ConnectionResetException ex2) {}
                finally {
                    this.impl.releaseFD();
                }
            }
            if (this.impl.isClosedOrPending()) {
                throw new SocketException("Socket closed");
            }
            if (this.impl.isConnectionResetPending()) {
                this.impl.setConnectionReset();
            }
            if (this.impl.isConnectionReset()) {
                throw new SocketException("Connection reset");
            }
            this.eof = true;
            return -1;
        }
    }
    
    @Override
    public int read() throws IOException {
        if (this.eof) {
            return -1;
        }
        this.temp = new byte[1];
        if (this.read(this.temp, 0, 1) <= 0) {
            return -1;
        }
        return this.temp[0] & 0xFF;
    }
    
    @Override
    public long skip(long n) throws IOException {
        if (n <= 0L) {
            return 0L;
        }
        int n2 = (int)Math.min(1024L, n);
        byte[] array = new byte[n2];
        while (n > 0L) {
            int read = this.read(array, 0, (int)Math.min((long)n2, n));
            if (read < 0) {
                break;
            }
            n -= (long)read;
        }
        return n - n;
    }
    
    @Override
    public int available() throws IOException {
        return this.impl.available();
    }
    
    @Override
    public void close() throws IOException {
        if (this.closing) {
            return;
        }
        this.closing = true;
        if (this.socket != null) {
            if (!this.socket.isClosed()) {
                this.socket.close();
            }
        }
        else {
            this.impl.close();
        }
        this.closing = false;
        return;
    }
    
    void setEOF(boolean eof) {
        this.eof = eof;
    }
    
    @Override
    protected void finalize() {
    }
    
    private static native void init();
    
    static {
        init();
    }
}
